/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.config.shared;

/**
 * Almost certainly the wrong way to do this, should be based on the lhs instead of the specified return type.
 */
public class CfgExpression extends AbstractCfgObject {
  public enum Op {
    PLUS, MINUS, MULTIPLY, DIVIDE;
    public static Op parse(String symbol) {
      if ("+".equals(symbol)) {
        return PLUS;
      } else if ("-".equals(symbol)) {
        return MINUS;
      } else if ("*".equals(symbol)) {
        return MULTIPLY;
      } else if ("/".equals(symbol)) {
        return DIVIDE;
      }
      throw new IllegalArgumentException("Not a valid operator " + symbol);
    }
  }
  private final AbstractCfgObject lhs,rhs;
  private final Op op;

  public CfgExpression(SourceInfo source, AbstractCfgObject lhs, AbstractCfgObject rhs, String op) {
    this(source, lhs, rhs, Op.parse(op));
  }

  public CfgExpression(SourceInfo source, AbstractCfgObject lhs, AbstractCfgObject rhs, Op op) {
    super(source);
    assert lhs != null : "lhs";
    assert rhs != null : "rhs";
    assert op != null : "op";
    this.lhs = lhs;
    this.rhs = rhs;
    this.op = op;
  }

  public AbstractCfgObject getLhs() {
    return lhs;
  }

  public AbstractCfgObject getRhs() {
    return rhs;
  }

  public Op getOp() {
    return op;
  }

  @Override
  public int resolveInt() {
    switch (op) {
      case PLUS:
        return lhs.resolveInt() + rhs.resolveInt();
      case MINUS:
        return lhs.resolveInt() - rhs.resolveInt();
      case MULTIPLY:
        return lhs.resolveInt() * rhs.resolveInt();
      case DIVIDE:
        return lhs.resolveInt() / rhs.resolveInt();
      default:
        throw new WrongTypeException("Operator cannot be applied to int: " + op + " " + getSource());
    }
  }

  @Override
  public double resolveDouble() {
    switch (op) {
      case PLUS:
        return lhs.resolveDouble() + rhs.resolveDouble();
      case MINUS:
        return lhs.resolveDouble() - rhs.resolveDouble();
      case MULTIPLY:
        return lhs.resolveDouble() * rhs.resolveDouble();
      case DIVIDE:
        return lhs.resolveDouble() / rhs.resolveDouble();
      default:
        throw new WrongTypeException("Operator cannot be applied to double: " + op + " " + getSource());
    }
  }

  @Override
  public String resolveString() {
    switch (op) {
      case PLUS:
        return lhs.resolveString() + rhs.resolveString();
      case MINUS:
      case MULTIPLY:
      case DIVIDE:
      default:
        throw new WrongTypeException("Operator cannot be applied to get a string: " + op + " " + getSource());
    }
  }

  @Override
  public CfgObject resolveObject() {
    throw new WrongTypeException("Can't turn expression into an Object " + getSource());
  }

  @Override
  public CfgArray resolveArray() {
    throw new WrongTypeException("Can't turn expression into an Array " + getSource());
  }

  @Override
  public void accept(CfgVisitor visitor) {
    visitor.visitExpression(this);
  }
}
