/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.config.shared;

public class CfgReference extends AbstractCfgObject {
  private final CfgObject parent;
  private final String reference;
  private final String name;

  private AbstractCfgObject resolved;

  public CfgReference(SourceInfo source, CfgObject parent, String reference, String name) {
    super(source);
    this.parent = parent;
    this.reference = reference;
    this.name = name;
  }

  public String getName() {
    return name;
  }

  public String getReference() {
    return reference;
  }

  @Override
  public int resolveInt() {
    return tryResolve().resolveInt();
  }

  @Override
  public double resolveDouble() {
    return tryResolve().resolveDouble();
  }

  @Override
  public String resolveString() {
    return tryResolve().resolveString();
  }

  @Override
  public CfgObject resolveObject() {
    return tryResolve().resolveObject();
  }

  @Override
  public CfgArray resolveArray() {
    return tryResolve().resolveArray();
  }

  public AbstractCfgObject tryResolve() {
    if (resolved == null) {
      resolved = parent.get(reference);
      //TODO this doesn't work right as long as expressions don't replace themselves too, plus need to null out the old one
//      parent.put(name, resolved);
      if (resolved == null) {
        throw new KeyNotFoundException("Unable to resolve " + reference + " for property" + name + " at " + getSource());
      }
    }
    return resolved;
  }

  @Override
  public void accept(CfgVisitor visitor) {
    visitor.visitReference(this);
  }
}
