/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package java.io;

import java.io.IOException;

/**
 * Emulation of the {@link java.io.InputStream} class for the GWT edition.
 *
 * @author Jerome Louvel
 */
public class InputStream {

  /** The next position to read. */
  private int position;

  /** The text to stream. */
  protected String text;

  public InputStream() {
    super();
  }

  /**
   * Constructor.
   *
   * @param text
   */
  public InputStream(String text) {
    super();
    this.position = 0;
    this.text = text;
  }

  /**
   *
   * @return
   * @throws IOException
   */
  public int available() throws IOException {
    if (text != null) {
      return text.length();
    }

    return 0;
  }

  public String getText() {
    return text;
  }

  public void setText(String text) {
    this.text = text;
  }

  public void close() throws IOException {

  }

  /**
   * Reads the next character in the source text.
   *
   * @return The next character or -1 if end of text is reached.
   * @throws IOException
   */
  public int read() throws IOException {
    return (this.position == this.text.length()) ? -1 : this.text
            .charAt(this.position++);
  }

  /**
   *
   * @param cbuf
   * @return
   * @throws IOException
   */
  public int read(char[] cbuf) throws IOException {
    return read(cbuf, 0, cbuf.length);
  }

  /**
   *
   * @param cbuf
   * @param off
   * @param len
   * @return
   * @throws IOException
   */
  public int read(char[] cbuf, int off, int len) throws IOException {
    if (position >= text.length())
      return -1;
    int n = Math.min(text.length() - position, len);
    text.getChars(position, position + n, cbuf, off);
    position += n;
    return n;
  }
}