/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.graphics.shared;

public class Css3GradientInfo {

  private final Color gradientColor;
  private final double percent;
  private final String percentString;

  private Css3GradientInfo(Color gradientColor, double percent, String percentString) {
    this.gradientColor = gradientColor;
    this.percent = percent;
    this.percentString = percentString;
  }

  public Color getColor() {
    return gradientColor;
  }

  public double getPercent() {
    return percent;
  }

  public String getPercentString() {
    return percentString;
  }

  public static Css3GradientInfo valueOf(String text) {
    if (text == null) {
      return null;
    }
    String[] gradientParts = text.split("\\s");

    String percentString;
    if (gradientParts.length == 1) {
      percentString = "999%";// invalid
    } else {
      percentString = gradientParts[1].trim();
    }
    double percent = Double.parseDouble(percentString.substring(0, percentString.length() - 1));
    return new Css3GradientInfo(
            Color.decode(gradientParts[0].trim()),
            percent,
            percentString
    );
  }
}
