/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.graphics.shared;

public class GraphicsUtil {
  public static String colorToHexString(Color inColor) {
    String rgb = Integer.toHexString(inColor.getRGB());
    return "#" + rgb.substring(2);
  }

  public static String rgbToHexString(int red, int green, int blue) {
    return colorToHexString(new Color(red, green, blue));
  }

  public static int[] rgbToHsl(int red, int green, int blue) {
    double max = Math.max(Math.max(red, green), blue); // 0xdd = 221
    double delta = max - Math.min(Math.min(red, green), blue); // 153
    double h = 0;
    int s = 0;
    int l = (int) Math.round(max * 100d / 255d); // 87 ok
    if (max != 0) {
      s = (int) Math.round(delta * 100d / max); // 69 ok
      if (max == red) {
        h = (green - blue) / delta;
      } else if (max == green) {
        h = (blue - red) / delta + 2d;
      } else {
        h = (red - green) / delta + 4d; // 4.8888888888
      }
      h = Math.min(Math.round(h * 60d), 360d); // 293
      if (h < 0d) {
        h += 360d;
      }
    }
    return new int[]{(int) Math.round(h), Math.round(s), l};
  }

  public static String colorNameToHexString(String colorName) {
    return colorToHexString(GraphicsUtilColorManager.getColorByName(colorName));
  }

  public static Color lighten(Color inColor, double inAmount) {
    int red = (int) ((inColor.getRed() * (1 - inAmount) / 255 + inAmount) * 255);
    int green = (int) ((inColor.getGreen() * (1 - inAmount) / 255 + inAmount) * 255);
    int blue = (int) ((inColor.getBlue() * (1 - inAmount) / 255 + inAmount) * 255);
    return new Color(red, green, blue);
  }

  public static Color darken(Color inColor, double inAmount) {
    int red = (int) (inColor.getRed() * (1 - inAmount));
    int green = (int) (inColor.getGreen() * (1 - inAmount));
    int blue = (int) (inColor.getBlue() * (1 - inAmount));
    return new Color(red, green, blue);
  }

  public static Color mix(Color a, Color b) {
    return mix(a, b, 0.5);
  }
  public static Color mix(Color a, Color b, double ratio) {
    int red = (int) (a.getRed() * (1.0 - ratio) + b.getRed() * ratio);
    int green = (int) (a.getGreen() * (1.0 - ratio) + b.getGreen() * ratio);
    int blue = (int) (a.getBlue() * (1.0 - ratio) + b.getBlue() * ratio);
    return new Color(red, green, blue);
  }

  public static String hslToHexString(int hue, int saturation, int lightness) {
    return hslToHexString(new int[]{hue, saturation, lightness});
  }

  public static String hslToHexString(int[] hsl) {
    int[] rgb = hslToRgb(hsl);
    return rgbToHexString(rgb[0], rgb[1], rgb[2]);
  }

  public static int[] hslToRgb(int[] hsl) {
    double hue = hsl[0] / 360d;
    double saturation = hsl[1] / 100d;
    double lightness = hsl[2] / 100d;
    double red = 0d;
    double green = 0d;
    double blue;

    if (saturation > 0d) {
      if (hue >= 1d) {
        hue = 0d;
      }

      hue = hue * 6d;
      double f = hue - Math.floor(hue);
      double a = Math.round(lightness * 255d * (1d - saturation));
      blue = Math.round(lightness * 255d * (1d - (saturation * f)));
      double c = Math.round(lightness * 255d * (1d - (saturation * (1d - f))));
      lightness = Math.round(lightness * 255d);

      switch ((int) Math.floor(hue)) {
        case 0:
          red = lightness;
          green = c;
          blue = a;
          break;
        case 1:
          red = blue;
          green = lightness;
          blue = a;
          break;
        case 2:
          red = a;
          green = lightness;
          blue = c;
          break;
        case 3:
          red = a;
          green = blue;
          blue = lightness;
          break;
        case 4:
          red = c;
          green = a;
          blue = lightness;
          break;
        case 5:
          red = lightness;
          green = a;
          break;
      }
      return new int[]{(int) Math.round(red), (int) Math.round(green), (int) Math.round(blue)};
    }

    lightness = Math.round(lightness * 255d);
    return new int[]{(int) lightness, (int) lightness, (int) lightness};
  }

  public static int[] hexStringToHsl(String color) {
    Color rgbColor = Color.decode(color);
    return rgbToHsl(rgbColor.getRed(), rgbColor.getGreen(), rgbColor.getBlue());
  }
}
