/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.slicer.client;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.GWT.UncaughtExceptionHandler;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArrayString;
import com.google.gwt.core.client.JsonUtils;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.InsertPanel;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.web.bindery.autobean.shared.AutoBean;
import com.google.web.bindery.autobean.shared.AutoBeanCodex;
import com.google.web.bindery.autobean.shared.AutoBeanFactory;
import com.google.web.bindery.autobean.shared.Splittable;
import com.sencha.gxt.themebuilder.config.client.impl.AbstractDetailsImpl;
import com.sencha.gxt.themebuilder.config.client.impl.ConfigParser;
import com.sencha.gxt.themebuilder.config.shared.CfgObject;
import com.sencha.gxt.themebuilder.config.shared.ParseException;
import com.sencha.gxt.themebuilder.config.shared.SourceInfo;
import com.sencha.gxt.themebuilder.config.shared.ThemerUtils;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData;
import com.sencha.gxt.themebuilder.base.client.config.ThemeDetails;


/**
 * Simple entrypoint to draw each widget that needs to be sliced, and to
 * expose details about how it should be sliced.
 */
public abstract class SliceTheme implements EntryPoint {
  private static final Logger logger = Logger.getLogger(SliceTheme.class.getName());

  public ThemeDetails getDetails() {
    return GWT.create(ThemeDetails.class);
  }

  public abstract List<Job> getJobs();

  interface Wrapper {
    List<WidgetData> getData();

    void setData(List<WidgetData> data);
  }

  public interface ABF extends AutoBeanFactory {
    AutoBean<Wrapper> wrapper();

    AutoBean<WidgetData> widget();
  }

  public static final ABF factory = GWT.create(ABF.class);

  private InsertPanel.ForIsWidget container;
  private SafeHtmlBuilder debugBuilder = new SafeHtmlBuilder();

  @Override
  public void onModuleLoad() {
    GWT.setUncaughtExceptionHandler(new UncaughtExceptionHandler() {
      @Override
      public void onUncaughtException(Throwable e) {
        logger.log(Level.SEVERE, "Uncaught Exception ", e);
      }
    });

    exportReadyForConfig();

    createManualConfigField();
  }

  private void createManualConfigField() {
    final FlowPanel panel = new FlowPanel();
    final List<TextArea> fields = new ArrayList<TextArea>();
    addFile(panel, fields);

    RootPanel.get().add(panel);
    RootPanel.get().add(new Button("Add new file", new ClickHandler() {
      @Override
      public void onClick(ClickEvent event) {
        addFile(panel, fields);
      }
    }));
    RootPanel.get().add(new Button("Build", new ClickHandler() {
      @Override
      public void onClick(ClickEvent event) {
        JsArrayString configs = JavaScriptObject.createArray().cast();
        for (TextArea field : fields) {
          configs.push(field.getValue());
        }
        try {
          readConfig(configs);
        } catch (ParseException ex) {
          Window.alert(ex.getMessage());
        }
      }
    }));
  }

  private void addFile(FlowPanel parent, List<TextArea> fields) {
    TextArea field = new TextArea();
    field.setPixelSize(800, 600);

    fields.add(field);
    parent.add(field);
  }

  protected native void exportReadyForConfig() /*-{
    var that = this;
    $wnd.readyForConfig = $entry(function(){
      that.@com.sencha.gxt.themebuilder.slicer.client.SliceTheme::readConfig(*)(Array.prototype.slice.apply(arguments));
    });
  }-*/;

  private void readConfig(JsArrayString config) throws ParseException {
    assert config.length() > 0 : "Need at least one config to parse!";
    CfgObject current = ConfigParser.parse(config.shift());
    while (config.length() != 0) {
      current = new ThemerUtils(SourceInfo.GENERATED, current).extend(current, ConfigParser.parse(config.shift()));
    }
    AbstractDetailsImpl.theme = current.get("theme").resolveObject();


    RootPanel.get().clear();

    // set up a list, and a wrapper
    final AutoBean<Wrapper> wrap = factory.wrapper();
    List<WidgetData> list = new ArrayList<WidgetData>();
    wrap.as().setData(list);


    VerticalPanel panel = new VerticalPanel();
    panel.setSpacing(10);
    container = panel;

    RootPanel.get().add((Widget) container);

    final List<Job> jobs = getJobs();
    addDebugInfo("jobs", jobs.size());


    for (Job job : jobs) {
      if (job.isSkipped()) {
        continue;
      }
      try {
        job.init(getDetails(), factory);

        job.go(container);
      } catch (Exception ex) {
        logger.log(Level.SEVERE, "Fatal error trying to start job " + job, ex);
        return;
      }
    }

    addDebugInfo("widgets", container.getWidgetCount());

    // be sure all widgets are ready before collecting manifest data
    Scheduler.get().scheduleDeferred(new Scheduler.ScheduledCommand() {
      @Override
      public void execute() {
        for (Job job : jobs) {
          if (job.isSkipped()) {
            continue;
          }
          try {
            job.collectWidgetData(new Job.JobCallback() {
              @Override
              public void addWidgetData(WidgetData result) {
                wrap.as().getData().add(result);
              }
            });
          } catch (RuntimeException ex) {
            logger.log(Level.SEVERE, "Fatal error trying to collect data on job " + job, ex);
            return;
          }
        }
        finished(wrap);
      }
    });
  }

  private void addDebugInfo(String key, Object value) {
    String v = "null";
    if (value != null) {
      v = value.toString();
    }

    debugBuilder.appendEscaped(key).appendEscaped(": ").appendEscaped(v).appendHtmlConstant("<br />");
  }

  /**
   * Indicates that the wrapper and all of its data are ready to be exported
   *
   * @param wrap the autobean holding the list of widgets
   */
  private void finished(AutoBean<Wrapper> wrap) {

    //container.insert(new HTML(debugBuilder.toSafeHtml()), 0);

    final JavaScriptObject wrapped;
    Splittable splittable = AutoBeanCodex.encode(wrap);
    if (GWT.isScript()) {
      // the splittable is a jso already
      wrapped = (JavaScriptObject) splittable;
    } else {
      // At present, the theming magic doesn't support dev mode, so this isn't too important, perf-wise
      wrapped = JsonUtils.unsafeEval(splittable.getPayload());
    }
    nativeNotifyAllWidgetsCreated(wrapped);
  }

  private native void nativeNotifyAllWidgetsCreated(JavaScriptObject wrapped) /*-{
      $wnd.slicerManifest = { "widgets": wrapped.data };
      $wnd.widgetsReady = true;
  }-*/;
}
