/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.slicer.client.job;

import com.google.gwt.dom.client.Document;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.InsertPanel;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.util.Rectangle;
import com.sencha.gxt.core.client.util.Util;
import com.sencha.gxt.themebuilder.slicer.client.Job;
import com.sencha.gxt.themebuilder.slicer.client.SliceTheme.ABF;
import com.sencha.gxt.themebuilder.slicer.client.data.BorderData;
import com.sencha.gxt.themebuilder.slicer.client.data.BoxData;
import com.sencha.gxt.themebuilder.slicer.client.data.RadiusData;
import com.sencha.gxt.themebuilder.slicer.client.data.Slices;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData;
import com.sencha.gxt.themebuilder.base.client.config.ThemeDetails;
import com.sencha.gxt.widget.core.client.Component;
import com.sencha.gxt.widget.core.client.event.XEvent;

public abstract class AbstractJob<W extends Widget> implements Job {
  protected ThemeDetails details;
  protected ABF factory;
  protected W widget;

  @Override
  public void init(ThemeDetails details, ABF factory) {
    this.details = details;
    this.factory = factory;
  }

  protected abstract W createWidget();

  @Override
  public void go(InsertPanel.ForIsWidget parent) {
    widget = createWidget();
    parent.add(new Label(this.getClass().getName()));
    parent.add(widget);
  }

  @Override
  public boolean isSkipped() {
    return false;
  }

  protected WidgetData createWidgetData(String hint) {
    WidgetData widgetData = factory.widget().as();
    widgetData.setId(getClass().getName() + "-" + hint);
    widgetData.setSprite(false);
    widgetData.setBorder(factory.create(BorderData.class).as());
    widgetData.setRadius(factory.create(RadiusData.class).as());
    widgetData.setBox(factory.create(BoxData.class).as());
    widgetData.setSlices(factory.create(Slices.class).as());

    return widgetData;
  }

  protected void setBox(Element element, BoxData box) {
    setBox(element.<XElement>cast(), box);
  }

  protected void setBox(XElement element, BoxData box) {
    Rectangle rect = element.getBounds();
    box.setX(rect.getX());
    box.setY(rect.getY());
    box.setW(rect.getWidth());
    box.setH(rect.getHeight());
  }

  protected void setBox(Widget w, BoxData box) {
    setBox(w.getElement(), box);
  }

  protected void setBorders(WidgetData data, int top, int right, int bottom, int left) {
    data.getBorder().setT(top);
    data.getBorder().setR(right);
    data.getBorder().setB(bottom);
    data.getBorder().setL(left);
  }

  protected void setBorders(WidgetData data, int border) {
    setBorders(data, border, border, border, border);
  }
  protected void setBorders(WidgetData data, XElement elt) {
    setBorders(data,
            Util.parseInt(elt.getComputedStyle("borderTopWidth"), 10),
            Util.parseInt(elt.getComputedStyle("borderRightWidth"), 10),
            Util.parseInt(elt.getComputedStyle("borderBottomWidth"), 10),
            Util.parseInt(elt.getComputedStyle("borderLeftWidth"), 10));  }

  protected void setRadius(WidgetData data, int tl, int tr, int br, int bl) {
    data.getRadius().setTl(tl);
    data.getRadius().setTr(tr);
    data.getRadius().setBr(br);
    data.getRadius().setBl(bl);
  }

  protected void setRadius(WidgetData data, int radius) {
    setRadius(data, radius, radius, radius, radius);
  }
  protected void setRadius(WidgetData data, XElement elt) {
    setRadius(data,
            Util.parseInt(elt.getComputedStyle("borderTopLeftRadius"), 10),
            Util.parseInt(elt.getComputedStyle("borderTopRightRadius"), 10),
            Util.parseInt(elt.getComputedStyle("borderBottomRightRadius"), 10),
            Util.parseInt(elt.getComputedStyle("borderBottomLeftRadius"), 10));
  }

  protected Event createMouseOverEvent(Widget widget) {
    return createMouseOverEvent(widget.getElement());
  }

  protected Event createMouseOverEvent(Element element) {
    return XEvent.as(Document.get().createMouseOverEvent(0, 0, 0, 0, 0, false, false, false, false, NativeEvent.BUTTON_LEFT, element));
  }

  protected Event createMouseClickEvent() {
    return XEvent.as(Document.get().createClickEvent(0, 0, 0, 0, 0, false, false, false, false));
  }

  protected void setSize(Component component) {
    component.setPixelSize(200, 200);
  }
}
