/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.slicer.client.job.field;

import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style.Unit;
import com.sencha.gxt.themebuilder.slicer.client.data.BoxData;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData;
import com.sencha.gxt.themebuilder.slicer.client.job.AbstractJob;
import com.sencha.gxt.widget.core.client.Slider;

/**
 *
 */
public class SliderJob extends AbstractJob<Slider> {

  protected static interface BoxUpdate {
    void run(BoxData boxData);
  }

  protected static final int SLIDER_SIZE = 90;

  @Override
  protected Slider createWidget() {
    return createWidget(false);
  }

  protected Slider createWidget(boolean vertical) {
    Slider slider = new Slider(vertical);

    if (vertical) {
      slider.setHeight(SLIDER_SIZE);
    } else {
      slider.setWidth(SLIDER_SIZE);
    }

    slider.setMinValue(0);
    slider.setValue(50);
    slider.setMaxValue(100);

    return slider;
  }


  public void collectWidgetData(JobCallback callback) {
    sliceThumb("", callback);

    sliceIt("HorizontalLeft", callback, new BoxUpdate() {
      @Override
      public void run(BoxData boxData) {
        boxData.setW(getCapSize());
      }
    });

    sliceIt("HorizontalRight", callback, new BoxUpdate() {
      @Override
      public void run(BoxData boxData) {
        int capSize = getCapSize();
        boxData.setX(boxData.getX() + (SLIDER_SIZE - capSize));
        boxData.setW(capSize);
      }
    });

    sliceIt("HorizontalMiddle", callback, new BoxUpdate() {
      @Override
      public void run(BoxData boxData) {
        boxData.setX(boxData.getX() + getCapSize());
        boxData.setW(2);
      }
    });
  }

  protected void sliceThumb(String suffix, JobCallback callback) {
    // ugly, but set the thumb outside
    Element thumb = widget.getCell().getAppearance().getThumb(widget.getElement());
    int newLocation = widget.getElement().getOffsetWidth() + thumb.getOffsetWidth() + 12;
    thumb.getStyle().setLeft(newLocation, Unit.PX);

    WidgetData thumbData = createWidgetData("thumb" + suffix);
    setBox(thumb, thumbData.getBox());
    thumbData.getSlices().setFull_capture("field/thumb" + suffix + ".png");
    callback.addWidgetData(thumbData);
  }


  protected void sliceIt(String suffix, JobCallback callback, BoxUpdate boxUpdate) {
    WidgetData widgetData = createWidgetData("track" + suffix);
    BoxData widgetDataBox = widgetData.getBox();
    setBox(widget, widgetDataBox);

    boxUpdate.run(widgetDataBox);

    widgetData.getSlices().setFull_capture("field/track" + suffix + ".png");
    callback.addWidgetData(widgetData);
  }

  private int getCapSize() {
    return Math.max(details.field().slider().thumbWidth() / 2, details.field().slider().trackRadius());
  }
}
