/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.slicer.client.job.listview;

import java.util.List;

import com.google.gwt.cell.client.AbstractCell;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.sencha.gxt.core.client.IdentityValueProvider;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.util.Util;
import com.sencha.gxt.data.shared.ListStore;
import com.sencha.gxt.data.shared.ModelKeyProvider;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData.GradientDir;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData.Side;
import com.sencha.gxt.themebuilder.slicer.client.job.AbstractJob;
import com.sencha.gxt.widget.core.client.ListView;

public class ListViewItemJob extends AbstractJob<ListView<String, String>> {

  @Override
  protected ListView<String, String> createWidget() {
    ListStore<String>store = new ListStore<String>(new ModelKeyProvider<String>() {
      @Override
      public String getKey(String item) {
        return item;
      }
    });
    store.add("item");
    store.add("itemover");
    store.add("itemselected");
    ListView<String, String> listview = new ListView<String, String>(store, new IdentityValueProvider<String>());
    listview.setCell(new AbstractCell<String>() {
      @Override
      public void render(Context context, String value, SafeHtmlBuilder sb) {
        sb.append(Util.NBSP_SAFE_HTML);
      }
    });

    return listview;
  }

  @Override
  public void collectWidgetData(JobCallback callback) {
    widget.getAppearance().onOver(widget.getElement(1), true);
    widget.getAppearance().onSelect(widget.getElement(2), true);
    List<String> all = widget.getStore().getAll();
    for (int i = 0; i < all.size(); i++) {
      String str = all.get(i);
      WidgetData data = createWidgetData(str);
      XElement elt = widget.getElement(i);

      setBorders(data, elt);
      setBox(elt, data.getBox());
      data.setGradient(GradientDir.bottom);
      data.setStretch(Side.BOTTOM);
      data.getSlices().setBg("listview/" + str + ".png");
      callback.addWidgetData(data);
    }
  }
}
